#include "port.h"
#include "utility.h"
#include "fang.h"
#include "fa_internal.h"

/*LINTLIBRARY*/

void fa_invert(geo, bbox, outgeo)
fa_geometry *geo;
fa_box *bbox;
fa_geometry *outgeo;
{
    fa_vertex *ptr, *loc;

    fa_sort(geo);

    fa_init(outgeo);
    fa_copy(geo, outgeo);

    if (bbox->left > outgeo->bb.left 
	    || bbox->bottom > outgeo->bb.bottom 
	    || bbox->right < outgeo->bb.right 
	    || bbox->top < outgeo->bb.top) {
	fa_error("Bounding box for inversion is smaller than the bounding box for the geometry");
    }

    outgeo->bb = *bbox;

    if (outgeo->head == NIL(fa_vertex)) {
	/* empty geometry */
	fa_add_box(outgeo, bbox->left, bbox->bottom, bbox->right, bbox->top);
	return;
    }

    for(ptr = outgeo->head; ptr != NIL(fa_vertex); ptr = ptr->next) {
	ptr->type = fa_inverse_type[(int) ptr->type];
    }

    /* add the lower edge of the bounding box. 
       Be careful to keep things properly sorted in the case
       where part of the original geometry touches the bbox */


    ptr = ALLOC(fa_vertex, 1);
    ptr->x = bbox->left;
    ptr->y = bbox->bottom;
    ptr->type = LOWER_LEFT_POS;
    ptr->next = outgeo->head; 
    outgeo->head = ptr;
    if (ptr->next == NIL(fa_vertex)) {
	outgeo->tail = ptr;
    }

    loc = outgeo->head;
    while (loc->next != NIL(fa_vertex) && loc->next->y == bbox->bottom) {
	loc = loc->next;
    }

    ptr = ALLOC(fa_vertex, 1);
    ptr->x = bbox->right;
    ptr->y = bbox->bottom;
    ptr->type = LOWER_RIGHT_POS;
    ptr->next = loc->next; 
    loc->next = ptr;
    if (loc == outgeo->tail) {
	outgeo->tail = ptr;
    }

	/* add the upper edge */

    loc = outgeo->head;
    while (loc->next != NIL(fa_vertex) && loc->next->y < bbox->top) {
	loc = loc->next;
    }

    ptr = ALLOC(fa_vertex, 1);
    ptr->x = bbox->left;
    ptr->y = bbox->top;
    ptr->type = UPPER_LEFT_POS;
    ptr->next = loc->next; 
    loc->next = ptr;
    if (loc == outgeo->tail) {
	outgeo->tail = ptr;
    }

    outgeo->count += 3;

    fa_put_vertex(outgeo, bbox->right, bbox->top, UPPER_RIGHT_POS);

    if (fa_debug) {
	(void) fprintf(stderr,"Invert, on output : \n");
	fa_dump_vertices(outgeo->head);
    }
}
