#include "port.h"
#include "utility.h"
#include "fang.h"
#include "scan.h"
#include "fa_internal.h"

/*LINTLIBRARY*/

typedef struct region_region region_region;
typedef struct region_point region_point;

struct region_region {
    fa_region region;
    region_point *head, *tail;
};

struct region_point {
    fa_coord x,y;
    int delta;
    region_region *region;
    region_point *next, *last;
};

typedef struct region_line region_line;

struct region_line {
    region_point *head, *tail;
};

#include "region_procs.h"	/* contains region_scan and 
				   region_check_emit */

void fa_to_regions(geo, outregion)
fa_geometry *geo;
fa_region **outregion;
{
    fa_vertex *vertex;
    region_line line;
    region_point *pointer;
    fa_coord current_y;
    int delta;

    fa_merge1(geo);

    *outregion = NIL(fa_region);
    
    region_init_line(&line, geo);

    vertex = geo->head;

    while ( vertex != NIL(fa_vertex) ) {
	
	current_y = vertex->y;
	pointer = line.head;

	while ( vertex != NIL(fa_vertex) && vertex->y == current_y) {
	    
	    delta = fa_vertex_delta( vertex->type );
	    pointer = region_scan(pointer, vertex->x);

	    if ( pointer->x == vertex->x ) {
		pointer->delta += delta;
	    } else {
		region_insert_point(pointer, vertex->x, vertex->y, delta);
		pointer = pointer->next;
	    }

	    region_check_emit( outregion, pointer, vertex );

	    if (pointer->delta == 0) {
		pointer = pointer->last;
		region_remove_point(pointer->next);
	    }

	    fa_next_vertex(&vertex);

	}

    }

    if ( line.head->next != line.tail ) {
	fa_error("Panic : Non-empty scan line at end of fa_to_region");
    }

    region_free_line(line);
}
