static char rcs_id[] = "$Id: bag.c,v 1.6 90/04/27 09:34:40 octtools Exp $";
/*
 * Implementations of the the oct functions for bags.
 */

#include "copyright.h"
#include "port.h"
#include "internal.h"
#include "io.h"

extern struct object_desc oct_default_desc;
static struct object_desc *super = &oct_default_desc;

static int bag_modify();
static int bag_free_fields();
static int bag_write_fields();
static int bag_read_fields();
static int bag_copy_fields();

oct_bag_desc_set(object_desc)
struct object_desc *object_desc;
{
    *object_desc = *super;
    
    object_desc->internal_size = sizeof(struct bag);
    object_desc->external_size = sizeof(struct octBag);
    object_desc->contain_mask = (octObjectMask) (OCT_ALL_MASK & ~OCT_FACET_MASK);
    object_desc->flags |= HAS_NAME;
    object_desc->user_offset = FIELD_OFFSET(struct bag, user_bag);
    object_desc->name_offset = FIELD_OFFSET(struct octBag, name),
    object_desc->copy_fields_func = bag_copy_fields;
    object_desc->modify_func = bag_modify;
    object_desc->free_fields_func = bag_free_fields;
    object_desc->write_fields_func = bag_write_fields;
    object_desc->read_fields_func = bag_read_fields;
}

/*ARGSUSED*/
static int
bag_copy_fields(o_p, c_p, size)
char *o_p, *c_p;
int size;
{
    struct octBag *orig = (struct octBag *) o_p;
    struct octBag *copy = (struct octBag *) c_p;

    *copy = *orig;

    if (copy->name != NIL(char)) {
	copy->name = oct_str_intern(copy->name);
    }
}

static int
bag_free_fields(ptr)
generic *ptr;
{
    struct bag *bag = (struct bag *) ptr;

    if (bag->user_bag.name != NIL(char)) {
	oct_str_free(bag->user_bag.name);
    }
}
    
static octStatus
bag_modify(ptr, obj)
generic *ptr;
struct octObject *obj;
{
    struct bag *bag = (struct bag *) ptr;
    char *new_name = obj->contents.bag.name;

    MODIFY_NAME(bag->user_bag.name, new_name);
    bag->user_bag.name = new_name;
    
    return OCT_OK;
}

static octStatus
bag_read_fields(ptr)
generic *ptr;
{
    struct bag *bag = (struct bag *) ptr;
    
    return (*super->read_fields_func)(ptr) &&
      oct_get_string(&bag->user_bag.name);
}

static octStatus
bag_write_fields(ptr)
generic *ptr;
{
    struct bag *bag = (struct bag *) ptr;
    
    return (*super->write_fields_func)(ptr) &&
      oct_put_string(bag->user_bag.name);
}
