#include "internal.h"
#include <stdio.h>

  /* The functions for opening a facet  */
  
octStatus oct_do_open(), oct_resolve(), oct_creat_facet(),
	  oct_open_facet_stream();

extern int errno;
extern int sys_nerr;
extern char *sys_errlist[];

#define sys_msg() (errno < sys_nerr ? sys_errlist[errno] : "Unknown error")

octStatus 
octOpenFacet(obj)
struct octObject *obj;
{
    struct facet *desc;
    struct octFacet *user_facet;
    octStatus retval;
    int mode;

    octBegin(); /* XXX: just in case, guarenteed to be safe */
    
    if (obj->type != OCT_FACET) {
	oct_error("octOpenFacet: The argument must be an octFacet object");
	return OCT_ERROR;
    }
    user_facet = &obj->contents.facet;

    /*XXX temporary compatibility hack */
    switch (user_facet->mode[0]) {
    case 'r':
	mode = OCT_READ;
	break;
    case 'w':
	mode = OCT_OVER_WRITE;
	break;
    case 'a':
	mode = OCT_APPEND;
	break;
    default:
	oct_error("Unknown mode for facet: (%s)", user_facet->mode);
	return OCT_ERROR;
    }
    switch (user_facet->mode[1]) {
    case '!':
	mode |= OCT_REVERT;
	break;
    case '\0':
	break;
    default:
	oct_error("Unknown mode for facet: (%s)", user_facet->mode);
	return OCT_ERROR;
    }
     
    if (user_facet->cell == NIL(char) || user_facet->view == NIL(char) ||
	    user_facet->facet == NIL(char)) {
	oct_error("octOpenFacet: Incompletely specified facet");
	return OCT_ERROR;
    }

    retval = oct_do_open(user_facet, &desc, mode, 1, NIL(struct facet));

    if (retval < OCT_OK) {
	oct_prepend_error("OctOpenFacet: ");
	return retval;
    }

    obj->objectId = desc->objectId;
    return retval;
}

octStatus 
octOpenMaster(iobj, fobj)
struct octObject *iobj;
struct octObject *fobj;
{
    struct octFacet *user_facet = &fobj->contents.facet;
    struct instance *inst;
    struct master *master;
    struct facet *master_desc;
    struct octFacet dummy;
    int mode;
    octStatus retval;

    inst = (struct instance *) oct_id_to_ptr(iobj->objectId);

    if (inst == NIL(struct instance)) {
	oct_error("OctOpenMaster: The arguments id has been corrupted");
	return OCT_CORRUPTED_OBJECT;
    }

    if (inst->flags.type != OCT_INSTANCE) {
	oct_error("octOpenMaster: The first argument must an octInstance object");
	return OCT_ERROR;
    }
    
    if (fobj->type != OCT_FACET) {
	oct_error("octOpenMaster: The second argument must an octFacet object");
	return OCT_ERROR;
    }


    /*XXX temporary compatibility hack */
    switch (user_facet->mode[0]) {
    case 'r':
	mode = OCT_READ;
	break;
    case 'w':
	mode = OCT_OVER_WRITE;
	break;
    case 'a':
	mode = OCT_APPEND;
	break;
    default:
	oct_error("Unknown mode for facet: (%s)", user_facet->mode);
	return OCT_ERROR;
    }
    switch (user_facet->mode[1]) {
    case '!':
	mode |= OCT_REVERT;
	break;
    case '\0':
	break;
    default:
	oct_error("Unknown mode for facet: (%s)", user_facet->mode);
	return OCT_ERROR;
    }

    master = inst->master;

    if (master->master && !strcmp(master->facet, user_facet->facet)) {
	master_desc = master->master;
	retval = oct_do_open_with_desc(master_desc, mode, 0);
	if (retval < OCT_OK) {
	    return retval;
	}
    } else {
	/* 
	 * For some reason, we couldn't open the master earlier,
	 * lets make one more try.
	 */
	dummy.cell = master->cell;
	dummy.view = master->view;
	dummy.facet = user_facet->facet;
	dummy.version = master->version;
	dummy.mode = user_facet->mode;

	retval = oct_do_open(&dummy, &master_desc, mode, 0, inst->facet);

	if (retval < OCT_OK) {
	    oct_prepend_error("OctOpenMaster: ");
	    return retval;
	} else if (retval == OCT_NEW_FACET) {
	    oct_error("OctOpenMaster: Cannot find master for instance %s",
		      oct_facet_name(&dummy));
	    return OCT_ERROR;
	}
    }

    *user_facet = master_desc->user_facet; 
    fobj->objectId = master_desc->objectId;
    
    return retval;
}

octStatus 
oct_do_open(user_facet, odesc, mode, create, parent)
struct octFacet *user_facet;
struct facet **odesc;
int mode, create;
struct facet *parent;
{
    struct facet *desc;
    char *file_desc;
    int need_to_free_desc = 0, retval;

    retval = oct_make_key(user_facet, &file_desc, parent);
    
    if (retval < OCT_OK) {
	return retval;
    }
    
    if (!oct_lookup_facet(file_desc, &desc)) {
	desc = ALLOC(struct facet);
	if (desc == NIL(struct facet)) {
	    oct_error("Out of Memory");
	    return OCT_ERROR;
	}
	oct_full_init_facet(user_facet, desc);
	need_to_free_desc = 1;
	
	retval = oct_save_key(file_desc, &desc->file_desc);
	if (retval != OCT_OK) {
	    FREE(desc);
	    return retval;
	}
    }

    /* 
     * At this point we have either existing facet or a properly 
     * initialized new one (include the file_desc from fsys).
     * oct_do_open_with_desc does the rest.
     */
    retval = oct_do_open_with_desc(desc, mode, create);

    if (retval < OCT_OK) {
	if (need_to_free_desc) {
	    FREE(desc);
	}
    } else {
	*odesc = desc;
    }

    return retval;
}

/* 
 * Do whatever is necessary to get all of desc into memory. Desc must 
 * at least be intialized and have a valid file_desc field 
 */
octStatus 
oct_do_open_with_desc(desc, mode, create)
struct facet *desc;
int mode, create;
{
    octStatus resolve_val, retval, final_val;
    FILE *file;
    int revert = mode & OCT_REVERT;
    int full_facet_in_memory = 0, just_info_in_memory = 0, not_in_memory = 0;

    if (desc->facet_flags.all_loaded) {
	full_facet_in_memory = 1;
    } else if (desc->facet_flags.info_loaded) {
	just_info_in_memory = 1;
    } else {
	not_in_memory = 1;
    }
    
    if (full_facet_in_memory && !revert) {
	/* we alread have what we need */
	if (mode == OCT_OVER_WRITE) {
	    oct_error(
	       "Cannot open for over-write a facet that is already in memory");
	    return OCT_ERROR;
	}
	desc->mode |= mode;
	desc->ref_count++;
	return OCT_ALREADY_OPEN;
    }

    if (full_facet_in_memory) {
	/* 
	 * We are reverting to the disk copy, so clear out the in 
	 * memory copy
	 */
	oct_free(desc);
	oct_free_info(desc);
	oct_init_facet_in_place(desc);
    } else if (just_info_in_memory) {
	/* 
	 * We have just the facet-info, free that copy and read the 
	 * full facet from disk (we don't try to do incremental reads, 
	 * it isnt worth the effort, so far.
	 */
	oct_free_info(desc);
	oct_init_facet_in_place(desc);
    }
    
    resolve_val = oct_resolve(&desc->user_facet, mode, desc->file_desc);

    if (resolve_val < OCT_OK) {
	return resolve_val;
    }
	
    if (resolve_val == OCT_NEW_FACET) {
	if (!create) {
	    return OCT_NEW_FACET;
	}

	retval = inherit_formals(desc);
	if (retval != OCT_OK) {
	    return retval;
	}
	final_val = OCT_NEW_FACET;

    } else if (mode == OCT_OVER_WRITE) {
	    /* we are over-writing the old copy */
	retval = inherit_formals(desc);
	if (retval != OCT_OK) {
	    return retval;
	}
	final_val = OCT_NEW_FACET;
    } else {
	if (oct_open_facet_stream(desc->file_desc, "r", &file) != OCT_OK) {
	    return OCT_ERROR;
	}

	if (!oct_restore(file, desc)) {
	    fclose(file);
	    oct_error("The disk representation is corrupted");
	    return OCT_ERROR;
	}
	fclose(file);

	final_val = OCT_OLD_FACET;

	retval = oct_check_masters_and_contents(desc);

	if (retval != OCT_OK) {
	    if (retval == OCT_INCONSISTENT) {
		final_val = retval;
	    } else {
		return retval;
	    }
	}
	(void) oct_bb_get((generic *) desc, NIL(struct octBox));
    }

    desc->facet_flags.all_loaded = desc->facet_flags.info_loaded = 1;

    if (not_in_memory) {
	oct_mark_facet(desc);
    }

    if (!full_facet_in_memory) {
	desc->objectId = oct_new_id((generic *)desc);
    }

    desc->mode |= mode;
    desc->ref_count++;
    return final_val;
}

/* 
 * Open the corresponding contents facet for desc and copy the formal 
 * terminals over.
 */

static octStatus
inherit_formals(desc)
struct facet *desc;
{
    int retval;

    if (strcmp(desc->user_facet.facet, "contents") != 0) {
	struct octFacet contents_user_facet;
	struct facet *contents_desc;
	struct term *ptr;
	
	contents_user_facet.cell = desc->user_facet.cell;
	contents_user_facet.view = desc->user_facet.view;
	contents_user_facet.facet = "contents";
	contents_user_facet.version = desc->user_facet.version;
	
	retval = oct_get_facet_info(&contents_user_facet,
				    &contents_desc, NIL(struct facet));
	
	if (retval == OCT_NEW_FACET) {
	    desc->contents_facet = NIL(struct facet);
	        /*XXX complain here? */
	    return OCT_OK;
	} else if (retval < OCT_OK) {
	    oct_prepend_error("Opening the corresponding contents facet: ");
	    return retval;
	}

	desc->contents_facet = contents_desc;
	DLL_ATTACH(contents_desc->interfaces, desc, struct facet,
		   next_interface, last_interface);

	ptr = (struct term *) contents_desc->object_lists[OCT_FORMAL];
	while (ptr != NIL(struct term)) {
	    retval = oct_add_interface_formal(desc, ptr, NIL(struct term *));
	    if (retval != OCT_OK) {
		return retval;
	    }
	    ptr = (struct term *) ptr->last;
	}
	desc->contents_date = contents_desc->formal_date;
    } else {
	desc->contents_facet = desc;
    }
    
    return OCT_OK;
}

/* 
 * Reset all the fields in desc except for the interfacet and 
 * instances lists. oct_free_info should have already been called.
 */ 
oct_init_facet_in_place(desc)
struct facet *desc;
{
    int i;

    desc->flags.type = OCT_FACET;
    desc->prop_list = desc->contents = desc->containers = NIL(struct chain);
    desc->generators = NIL(struct generator);
    desc->search_path = NIL(char *);
    desc->facet = desc;
    desc->object_count = 0;
    desc->temp_count = 0;
    desc->function_mask = (octFunctionMask) 0;
    desc->object_mask = (octObjectMask) 0;
    desc->num_formals = 0;
    CLEAR_FLAGS(desc);
    CLEAR_FACET_FLAGS(desc);
    desc->next_xid = 1;
    desc->xid_table = oct_make_xid_table((int32)0);
    desc->externalId = oct_next_xid(desc, (generic *) desc);
    desc->masters = oct_make_master_table();
    desc->next_interface = desc->last_interface = NIL(struct facet);
    desc->bb_date = desc->formal_date = desc->contents_date =
        desc->create_date = desc->delete_date = desc->modify_date =
	desc->attach_date = desc->detach_date = desc->time_stamp = 0;

    for(i=0; i<=OCT_MAX_TYPE; i++) {
	desc->object_lists[i] = (generic *) 0;
    }
}

/* 
 * Reset everything in the facet, should only be used on a new facet, 
 * not one that is already in memory.
 */
oct_full_init_facet(user_facet, desc)
struct octFacet *user_facet;
struct facet *desc;
{
    oct_init_facet_in_place(desc);
    desc->user_facet.cell = strsave(user_facet->cell);
    desc->user_facet.view = strsave(user_facet->view);
    desc->user_facet.facet = strsave(user_facet->facet);
    desc->user_facet.version = strsave(user_facet->version);
    desc->instantiations = NIL(struct master);
    desc->interfaces = NIL(struct facet);
    desc->mode = 0;
    desc->ref_count = 0;
}


octStatus octCopyFacet(new, old)
struct octObject *new;
struct octObject *old;
{
    struct octFacet *n_facet, *o_facet;
    char *n_desc, *o_desc;
    FILE *n_file, *o_file;
    int c, retval;

    if (new->type != OCT_FACET || old->type != OCT_FACET) {
	oct_error("octCopyFacet: both arguments must be facets");
	return OCT_ERROR;
    }

    n_facet = &new->contents.facet;
    if (n_facet->cell == NIL(char) || n_facet->view == NIL(char) ||
	    n_facet->facet == NIL(char)) {
	oct_error("octCopyFacet: Incompletely specified new facet");
	return OCT_ERROR;
    }
    
    o_facet = &old->contents.facet;
    if (o_facet->cell == NIL(char) || o_facet->view == NIL(char) ||
	    o_facet->facet == NIL(char)) {
	oct_error("octCopyFacet: Incompletely specified old facet");
	return OCT_ERROR;
    }
    
    retval = oct_make_key(n_facet, &n_desc, NIL(struct facet));
    if (retval < 0) {
	oct_prepend_error("octCopyFacet: ");
	return retval;
    }
    
    retval = oct_resolve(n_facet, OCT_OVER_WRITE, n_desc);
    if (retval < 0) {
	oct_prepend_error("octCopyFacet: ");
	return retval;
    }
    
    if (oct_open_facet_stream(n_desc, "w", &n_file) != OCT_OK) {
	oct_prepend_error("octCopyFacet: ");
	return OCT_ERROR;
    }
    
    retval = oct_make_key(o_facet, &o_desc, NIL(struct facet));
    if (retval < 0) {
	oct_prepend_error("octCopyFacet: ");
	return retval;
    }
    
    retval = oct_resolve(o_facet, OCT_READ, o_desc);
    if (retval < 0) {
	oct_prepend_error("octCopyFacet: ");
	return retval;
    }
    
    if (oct_open_facet_stream(o_desc, "r", &o_file) != OCT_OK) {
	oct_prepend_error("octCopyFacet: ");
	return OCT_ERROR;
    }

    while ((c = getc(o_file)) != EOF) {
	if (putc(c,n_file) == EOF) {
	    oct_prepend_error("octCopyFacet: %s: ", sys_msg());
	    return OCT_ERROR;
	}
    }
    fclose(o_file); fclose(n_file);
    return OCT_OK;
}
