static char rcs_id[] = "$Id: ref.c,v 1.6 90/04/27 09:33:10 octtools Exp $";
#include "copyright.h"
#include "port.h"
#include "internal.h"

static int gen_fix();

/* 
 * To avoid dangling pointers in generators, each object keeps a 
 * reference count of generators that current point toward it.  When 
 * an object with a non-zero reference count is deleted or detached, a
 * list of all active generators in the object's facet is searched for generators 
 * that point to that object.  Those generators are moved past the 
 * offending object so no dangling references are left.  The reference 
 * count lets us search that list of active generators only for 
 * objects that are actually being referenced. Since it is
 * unlikely that several generators will be simultaneously pointing to 
 * the same object, the ref_count field is only 2 bits deep.  If the 
 * count overflows that field (i.e. becomes larger than REF_MANY), it 
 * is set permanently to REF_MANY.  In this rare case, there will be a 
 * perhaps un-neccessary search of the active generators when that 
 * object is deleted  or detached
 */

#ifdef notdef  

#define REF_MANY 3

oct_ref_increment(obj)
generic *obj;
{
    int val = obj->flags.ref_count;

    if (++val > REF_MANY) {
	val = REF_MANY;
    }

    obj->flags.ref_count = val;
    return 1;
}

oct_ref_decrement(obj)
generic *obj;
{
    int val = obj->flags.ref_count;

    if (val == REF_MANY) {
	return 1;
    }
    
    if (--val < 0) {
	oct_error("Panic: negative generator reference count");
	/* XXX sometimes it is allowed to go negative! */
	/* errRaise(OCT_PKG_NAME, OCT_ERROR, octErrorString()); */
	return 0;
    }
    
    obj->flags.ref_count = val;
    return 1;
}
#endif /* notdef */

/* 
 * Fix all the generators that reference `obj' through `link'. If 
 * `link' is nil, then fix any generator that references obj through 
 * any link (this is the case where obj is being deleted rather than 
 * just detached)
 */

oct_fix_generators(link, obj)
struct chain *link;
generic *obj;
{
    struct generator *ptr;

    ptr = obj->facet->generators;

    while (ptr != NIL(struct generator)) {
	if (!gen_fix(ptr, link, obj)) {
	    return 0;
	}
	ptr = ptr->last_gen;
    }
    return 1;
}


#define DO_GEN_FIX(generator) (*generator->fix_func)(generator)

static int gen_fix(generator, link, obj)
struct generator *generator;
struct chain *link;
generic *obj;
{
    if (link == NIL(struct chain)) {
	while (generator->next && (generator->next->object == obj ||
				   generator->next->container == obj)) {
	    DO_GEN_FIX(generator);
	}
    } else while (generator->next == link) {
	DO_GEN_FIX(generator);
    }
    return 1;
}

