#include "port.h"
#include "fang.h"

/*
 *    Works like "squares" command in magic's cifoutput specification
 */

void
squares(geo, border, size, sep, outgeo)
fa_geometry    *geo;        /* Input geo passed in */
int        border;        /* Min. border size if possible */
int        size;        /* Square size */
int        sep;        /* Distance between squares */
fa_geometry    *outgeo;    /* Pointer to output geo created */
{
    fa_box        *boxes, *box;
    int        nx, ny;
    int        x, y;
    int        x_start, nxn;

    fa_init(outgeo);
    fa_to_boxes(geo, &boxes);
    
    for (box = boxes; box; box = box->next) {
        if (size > box->right - box->left)
            continue;
        if (size > box->top - box->bottom)
            continue;

        nx = (box->right - box->left - 2 * border + sep) / (size + sep);
        ny = (box->top - box->bottom - 2 * border + sep) / (size + sep);

        x_start = box->left + (box->right - box->left -
                nx * size - (nx-1) * sep)/2;

        y = box->bottom + (box->top - box->bottom -
                ny * size - (ny-1) * sep)/2;
        
        while (ny-- > 0) {
            x = x_start;
            nxn = nx;

            while (nxn-- > 0) {
                fa_add_box(outgeo, x, y, x + size, y + size);
                x += size + sep;
            }
            y += size + sep;
        }
    }
    fa_free_boxes(boxes);
    return;
}
