#include "port.h"
#include "copyright.h"
#include "utility.h"
#include "oct.h"
#include "region.h"
#include "oh.h"
#include "fang.h"
#include "harpoon.h"
#include "tap.h"
#include "th.h"
#include "pepint.h"

main(argc, argv)
int argc;
char **argv;
{
    octObject net1, net2;
    octObject term;
    octObject layer;
    octObject  facet;
    char  name[128];

    errProgramName(argv[0]);
    
    if((argc < 2) || (argc > 3)) {
	fprintf(stderr, "Usage:  %s  net1 [net2] | term\n", argv[0]);
	fprintf(stderr, "With one argument:  capacitance to ground.\n");
	fprintf(stderr, " (if the argument is not a net, it must be a term)\n");
	fprintf(stderr, "With two arguments: coupling capacitance.\n");
	exit(-1);
    }
    ohOpenFacet(&facet, "mytest", "symbolic", "contents", "r");
    thTechnology(&facet);
    
    if(argc == 3) {
	OH_ASSERT(ohGetByNetName(&facet, &net1, argv[1]));
	OH_ASSERT(ohGetByNetName(&facet, &net2, argv[2]));
	printf("Capacitance between %s and %s: %sF\n", argv[1], argv[2],
	       util_pretty_print(pepCrossCapacitance(&net1, &net2)));
    } else {
	if(ohGetByNetName(&facet, &net1, argv[1]) == OCT_OK) {
	    printf("Capacitance of net %s to ground: %sF\n", argv[1],
		   util_pretty_print(pepCapacitance(&net1)));
	} else if(ohGetByTermName(&facet, &term, argv[1]) == OCT_OK) {
	    printf("Capacitance of term %s to ground: %sF\n", argv[1],
		   util_pretty_print(pepTermCap( &term )));
	}
    }
    octCloseFacet(&facet);
    exit(0);
}
