/*LINTLIBRARY*/
#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "errtrap.h"
#include "oct.h"
#include "oh.h"
#include "tap.h"
#include "symlib.h"
#include "symlib_int.h"


void
symCreateFormalTerm(fct, term, name)
octObject *fct, *term;
char *name;
/*
 * create a formal terminal, making sure that a formal terminal of the
 * same name does not already exist
 */
{
    octStatus status;
    
    term->type = OCT_TERM;
    term->contents.term.instanceId = oct_null_id;
    term->contents.term.width = 1;
    if (name != NIL(char)) {
	status = ohGetByTermName(fct, term, name);
	switch (status) {
	    case OCT_OK:
	    errRaise(SYMLIB_PKG_NAME, SYM_ERROR, "formal terminal %s already exists",
		     term->contents.term.name);

	    case OCT_NOT_FOUND:
	    OCT_CHECK(octCreate(fct, term));
	    break;

	    default:
	    RAISE_ERROR;
	}
    } else {
	errRaise(SYMLIB_PKG_NAME, SYM_ERROR, "formal terminal name can not be NIL");
    }
    
    return;
}


void
symImplementFormalTerm(ft, at)
octObject *ft, *at;
/*
 * implement a formal terminal, making sure that it is not already implemented,
 * and making sure that the actual terminal does not already implement a formal.
 */
{
    octObject dummy, net;
    octStatus status;

    status = octGenFirstContent(ft, OCT_TERM_MASK, &dummy);
    switch (status) {
	case OCT_OK:
	errRaise(SYMLIB_PKG_NAME, SYM_ERROR, "formal terminal %s already implemented",
		 ft->contents.term.name);

	case OCT_GEN_DONE:
	break;

	default:
	RAISE_ERROR;
    }

    status = octGenFirstContainer(at, OCT_TERM_MASK, &dummy);
    switch (status) {
	case OCT_OK:
	errRaise(SYMLIB_PKG_NAME, SYM_ERROR, "actual terminal %s already implements a formal (%s)",
		 at->contents.term.name, dummy.contents.term.name);

	case OCT_GEN_DONE:
	break;

	default:
	RAISE_ERROR;
    }

    /* to stop symGetNet from returning a bogus new net */
    symGetNet(at, &net);
    OCT_CHECK(octAttach(ft, at));
    symAddTermToNet(ft, &net);
    
    return;
}
