/*LINTLIBRARY*/
#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "errtrap.h"
#include "st.h"
#include "oct.h"
#include "oh.h"
#include "symlib.h"
#include "symlib_int.h"


octStatus
symGetProperty(object, prop, name)
octObject *object, *prop;
char *name;
{
    octStatus status;
    octObject inst, master, obj;
    
    status = ohGetByPropName(object, prop, name);
    switch (status) {
	case OCT_OK:
	return OCT_OK;

	case OCT_NOT_FOUND:
	if (object->type == OCT_TERM) {
	    inst.objectId = object->contents.term.instanceId;
	    OCT_CHECK(octGetById(&inst));
	} else {
	    inst = *object;
	}
	OCT_CHECK(ohOpenMaster(&master, &inst, "interface", "r"));
	if (object->type == OCT_TERM) {
	    status = ohGetTerminal(&master, object->contents.term.formalExternalId, &obj);
	    switch (status) {
		case OCT_NOT_FOUND:
		errRaise(SYMLIB_PKG_NAME, SYM_ERROR, "master/instance inconsistency: terminal %s is not in the master",
			 object->contents.term.name);

		case OCT_OK:
		break;

		default:
		RAISE_ERROR;
	    }
	} else {
	    obj = master;
	}

	status = ohGetByPropName(&obj, prop, name);
	switch (status) {
	    case OCT_OK:
	    return OCT_OK;
	    
	    case OCT_NOT_FOUND:
	    return OCT_NOT_FOUND;

	    default:
	    RAISE_ERROR;
	}
	
	default:
	RAISE_ERROR;
    }
    return OCT_OK;
}


static octStatus
sym_prop_gen(state_p, obj)
char **state_p;
octObject *obj;
{
    st_generator *stgen = (st_generator *) *state_p;
    int status;
    char *key, *value;
    
    status = st_gen(stgen, &key, &value);
    if (status == 0) {
	return OCT_GEN_DONE;
    }
    *obj = *((octObject *) value);
    return OCT_OK;
}


static octStatus
sym_prop_free(state)
char *state;
{
    (void) st_free_gen((st_generator *) state);
    return OCT_OK;
}


void
symInitPropertyGen(object, gen)
octObject *object;
octGenerator *gen;
{
    octObject master, instance, obj, *prop;
    octGenerator pgen;
    octStatus status;
    st_table *tbl = st_init_table(strcmp, st_strhash);
    st_generator *stgen;

    if (object->type == OCT_TERM) {
	instance.objectId = object->contents.term.instanceId;
	OCT_CHECK(octGetById(&instance));
    } else {
	instance = *object;
    }
    OCT_CHECK(ohOpenMaster(&master, &instance, "interface", "r"));

    if (object->type == OCT_TERM) {
	status = ohGetTerminal(&master, object->contents.term.formalExternalId, &obj);
	switch (status) {
	    case OCT_NOT_FOUND:
	    errRaise(SYMLIB_PKG_NAME, SYM_ERROR, "master/instance inconsistency: terminal %s is not in the master",
		     object->contents.term.name);

	    case OCT_OK:
	    break;

	    default:
	    RAISE_ERROR;
	}
    } else {
	obj = master;
    }

    OCT_CHECK(octInitGenContents(&obj, OCT_PROP_MASK, &pgen));
    prop = ALLOC(octObject, 1);
    while ((status = octGenerate(&pgen, prop)) == OCT_OK) {
	(void) st_insert(tbl, prop->contents.prop.name, (char *) prop);
	prop = ALLOC(octObject, 1);
    }
    (void) free((char *) prop);

    if (status != OCT_GEN_DONE) {
	RAISE_ERROR;
    }
    
    OCT_CHECK(octInitGenContents(object, OCT_PROP_MASK, &pgen));
    prop = ALLOC(octObject, 1);
    while ((status = octGenerate(&pgen, prop)) == OCT_OK) {
	(void) st_insert(tbl, prop->contents.prop.name, (char *) prop);
	prop = ALLOC(octObject, 1);
    }
    (void) free((char *) prop);

    if (status != OCT_GEN_DONE) {
	RAISE_ERROR;
    }

    stgen = st_init_gen(tbl);
    OCT_CHECK(octInitUserGen((char *) stgen, sym_prop_gen, sym_prop_free, gen));
    
    return;
}    
